/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(NETSCAPE_PLUGIN_API)

#include "ArgumentCoders.h"
#include "Connection.h"
#include "MessageNames.h"
#include "PluginProcessMessagesReplies.h"
#include <wtf/Forward.h>
#include <wtf/ThreadSafeRefCounted.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebKit {
struct PluginProcessCreationParameters;
}

namespace Messages {
namespace PluginProcess {

static inline IPC::ReceiverName messageReceiverName()
{
    return IPC::ReceiverName::PluginProcess;
}

class InitializePluginProcess {
public:
    using Arguments = std::tuple<const WebKit::PluginProcessCreationParameters&>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProcess_InitializePluginProcess; }
    static constexpr bool isSync = false;

    explicit InitializePluginProcess(const WebKit::PluginProcessCreationParameters& processCreationParameters)
        : m_arguments(processCreationParameters)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CreateWebProcessConnection {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProcess_CreateWebProcessConnection; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetSitesWithData {
public:
    using Arguments = std::tuple<uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProcess_GetSitesWithData; }
    static constexpr bool isSync = false;

    explicit GetSitesWithData(uint64_t callbackID)
        : m_arguments(callbackID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DeleteWebsiteData {
public:
    using Arguments = std::tuple<const WallTime&, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProcess_DeleteWebsiteData; }
    static constexpr bool isSync = false;

    DeleteWebsiteData(const WallTime& modifiedSince, uint64_t callbackID)
        : m_arguments(modifiedSince, callbackID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DeleteWebsiteDataForHostNames {
public:
    using Arguments = std::tuple<const Vector<String>&, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProcess_DeleteWebsiteDataForHostNames; }
    static constexpr bool isSync = false;

    DeleteWebsiteDataForHostNames(const Vector<String>& hostNames, uint64_t callbackID)
        : m_arguments(hostNames, callbackID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(COCOA)
class SetQOS {
public:
    using Arguments = std::tuple<const int&, const int&>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProcess_SetQOS; }
    static constexpr bool isSync = false;

    SetQOS(const int& latencyQOS, const int& throughputQOS)
        : m_arguments(latencyQOS, throughputQOS)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

} // namespace PluginProcess
} // namespace Messages

#endif // ENABLE(NETSCAPE_PLUGIN_API)
