/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(NETSCAPE_PLUGIN_API)
#include "PluginProcessProxy.h"

#include "ArgumentCoders.h"
#include "Attachment.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "PluginProcessProxyMessages.h"
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace PluginProcessProxy {

#if PLATFORM(COCOA)

void LaunchProcess::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool result)
{
    encoder.get() << result;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

#if PLATFORM(COCOA)

void LaunchApplicationAtURL::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool result)
{
    encoder.get() << result;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

#if PLATFORM(COCOA)

void OpenURL::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool result, int32_t status, const String& launchedURLString)
{
    encoder.get() << result;
    encoder.get() << status;
    encoder.get() << launchedURLString;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

#if PLATFORM(COCOA)

void OpenFile::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, bool result)
{
    encoder.get() << result;
    connection.sendSyncReply(WTFMove(encoder));
}

#endif

} // namespace PluginProcessProxy

} // namespace Messages

namespace WebKit {

void PluginProcessProxy::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    Ref protectedThis { *this };
    if (decoder.messageName() == Messages::PluginProcessProxy::DidCreateWebProcessConnection::name())
        return IPC::handleMessage<Messages::PluginProcessProxy::DidCreateWebProcessConnection>(connection, decoder, this, &PluginProcessProxy::didCreateWebProcessConnection);
    if (decoder.messageName() == Messages::PluginProcessProxy::DidGetSitesWithData::name())
        return IPC::handleMessage<Messages::PluginProcessProxy::DidGetSitesWithData>(connection, decoder, this, &PluginProcessProxy::didGetSitesWithData);
    if (decoder.messageName() == Messages::PluginProcessProxy::DidDeleteWebsiteData::name())
        return IPC::handleMessage<Messages::PluginProcessProxy::DidDeleteWebsiteData>(connection, decoder, this, &PluginProcessProxy::didDeleteWebsiteData);
    if (decoder.messageName() == Messages::PluginProcessProxy::DidDeleteWebsiteDataForHostNames::name())
        return IPC::handleMessage<Messages::PluginProcessProxy::DidDeleteWebsiteDataForHostNames>(connection, decoder, this, &PluginProcessProxy::didDeleteWebsiteDataForHostNames);
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProcessProxy::SetModalWindowIsShowing::name())
        return IPC::handleMessage<Messages::PluginProcessProxy::SetModalWindowIsShowing>(connection, decoder, this, &PluginProcessProxy::setModalWindowIsShowing);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProcessProxy::SetFullscreenWindowIsShowing::name())
        return IPC::handleMessage<Messages::PluginProcessProxy::SetFullscreenWindowIsShowing>(connection, decoder, this, &PluginProcessProxy::setFullscreenWindowIsShowing);
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

bool PluginProcessProxy::didReceiveSyncMessage(IPC::Connection& connection, IPC::Decoder& decoder, UniqueRef<IPC::Encoder>& replyEncoder)
{
    Ref protectedThis { *this };
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProcessProxy::LaunchProcess::name())
        return IPC::handleMessageSynchronous<Messages::PluginProcessProxy::LaunchProcess>(connection, decoder, replyEncoder, this, &PluginProcessProxy::launchProcess);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProcessProxy::LaunchApplicationAtURL::name())
        return IPC::handleMessageSynchronous<Messages::PluginProcessProxy::LaunchApplicationAtURL>(connection, decoder, replyEncoder, this, &PluginProcessProxy::launchApplicationAtURL);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProcessProxy::OpenURL::name())
        return IPC::handleMessageSynchronous<Messages::PluginProcessProxy::OpenURL>(connection, decoder, replyEncoder, this, &PluginProcessProxy::openURL);
#endif
#if PLATFORM(COCOA)
    if (decoder.messageName() == Messages::PluginProcessProxy::OpenFile::name())
        return IPC::handleMessageSynchronous<Messages::PluginProcessProxy::OpenFile>(connection, decoder, replyEncoder, this, &PluginProcessProxy::openFile);
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return false;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled synchronous message %s to %" PRIu64, description(decoder.messageName()), decoder.destinationID());
    return false;
}

} // namespace WebKit

#endif // ENABLE(NETSCAPE_PLUGIN_API)
