/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if ENABLE(NETSCAPE_PLUGIN_API)

#include "ArgumentCoders.h"
#include "Connection.h"
#include "MessageNames.h"
#include "PluginProxyMessagesReplies.h"
#include <wtf/Forward.h>
#include <wtf/ThreadSafeRefCounted.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace WebCore {
class HTTPHeaderMap;
class IntRect;
class ProtectionSpace;
}

namespace WebKit {
class NPVariantData;
}

namespace Messages {
namespace PluginProxy {

static inline IPC::ReceiverName messageReceiverName()
{
    return IPC::ReceiverName::PluginProxy;
}

class LoadURL {
public:
    using Arguments = std::tuple<uint64_t, const String&, const String&, const String&, const WebCore::HTTPHeaderMap&, const Vector<uint8_t>&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_LoadURL; }
    static constexpr bool isSync = false;

    LoadURL(uint64_t requestID, const String& method, const String& urlString, const String& target, const WebCore::HTTPHeaderMap& headerFields, const Vector<uint8_t>& httpBody, bool allowPopups)
        : m_arguments(requestID, method, urlString, target, headerFields, httpBody, allowPopups)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class Update {
public:
    using Arguments = std::tuple<const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_Update; }
    static constexpr bool isSync = false;

    explicit Update(const WebCore::IntRect& paintedRect)
        : m_arguments(paintedRect)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ProxiesForURL {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_ProxiesForURL; }
    static constexpr bool isSync = true;

    using DelayedReply = ProxiesForURLDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const String& proxiesString);
    using Reply = std::tuple<String&>;
    using ReplyArguments = std::tuple<String>;
    explicit ProxiesForURL(const String& urlString)
        : m_arguments(urlString)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CookiesForURL {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_CookiesForURL; }
    static constexpr bool isSync = true;

    using DelayedReply = CookiesForURLDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const String& cookieString);
    using Reply = std::tuple<String&>;
    using ReplyArguments = std::tuple<String>;
    explicit CookiesForURL(const String& urlString)
        : m_arguments(urlString)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetCookiesForURL {
public:
    using Arguments = std::tuple<const String&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_SetCookiesForURL; }
    static constexpr bool isSync = false;

    SetCookiesForURL(const String& urlString, const String& cookieString)
        : m_arguments(urlString, cookieString)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetAuthenticationInfo {
public:
    using Arguments = std::tuple<const WebCore::ProtectionSpace&>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_GetAuthenticationInfo; }
    static constexpr bool isSync = true;

    using DelayedReply = GetAuthenticationInfoDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool returnValue, const String& username, const String& password);
    using Reply = std::tuple<bool&, String&, String&>;
    using ReplyArguments = std::tuple<bool, String, String>;
    explicit GetAuthenticationInfo(const WebCore::ProtectionSpace& protectionSpace)
        : m_arguments(protectionSpace)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetPluginElementNPObject {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_GetPluginElementNPObject; }
    static constexpr bool isSync = true;

    using DelayedReply = GetPluginElementNPObjectDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, uint64_t pluginElementNPObjectID);
    using Reply = std::tuple<uint64_t&>;
    using ReplyArguments = std::tuple<uint64_t>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class Evaluate {
public:
    using Arguments = std::tuple<const WebKit::NPVariantData&, const String&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_Evaluate; }
    static constexpr bool isSync = true;

    using DelayedReply = EvaluateDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool returnValue, const WebKit::NPVariantData& resultData);
    using Reply = std::tuple<bool&, WebKit::NPVariantData&>;
    using ReplyArguments = std::tuple<bool, WebKit::NPVariantData>;
    Evaluate(const WebKit::NPVariantData& npObjectAsVariantData, const String& scriptString, bool allowPopups)
        : m_arguments(npObjectAsVariantData, scriptString, allowPopups)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CancelStreamLoad {
public:
    using Arguments = std::tuple<uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_CancelStreamLoad; }
    static constexpr bool isSync = false;

    explicit CancelStreamLoad(uint64_t streamID)
        : m_arguments(streamID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ContinueStreamLoad {
public:
    using Arguments = std::tuple<uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_ContinueStreamLoad; }
    static constexpr bool isSync = false;

    explicit ContinueStreamLoad(uint64_t streamID)
        : m_arguments(streamID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CancelManualStreamLoad {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_CancelManualStreamLoad; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetStatusbarText {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_SetStatusbarText; }
    static constexpr bool isSync = false;

    explicit SetStatusbarText(const String& statusbarText)
        : m_arguments(statusbarText)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(COCOA)
class PluginFocusOrWindowFocusChanged {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_PluginFocusOrWindowFocusChanged; }
    static constexpr bool isSync = false;

    explicit PluginFocusOrWindowFocusChanged(bool pluginHasFocusAndWindowHasFocus)
        : m_arguments(pluginHasFocusAndWindowHasFocus)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class SetComplexTextInputState {
public:
    using Arguments = std::tuple<uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_SetComplexTextInputState; }
    static constexpr bool isSync = false;

    explicit SetComplexTextInputState(uint64_t complexTextInputState)
        : m_arguments(complexTextInputState)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class SetLayerHostingContextID {
public:
    using Arguments = std::tuple<uint32_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_SetLayerHostingContextID; }
    static constexpr bool isSync = false;

    explicit SetLayerHostingContextID(uint32_t layerHostingContextID)
        : m_arguments(layerHostingContextID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(X11)
class CreatePluginContainer {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_CreatePluginContainer; }
    static constexpr bool isSync = true;

    using DelayedReply = CreatePluginContainerDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, uint64_t windowID);
    using Reply = std::tuple<uint64_t&>;
    using ReplyArguments = std::tuple<uint64_t>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(X11)
class WindowedPluginGeometryDidChange {
public:
    using Arguments = std::tuple<const WebCore::IntRect&, const WebCore::IntRect&, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_WindowedPluginGeometryDidChange; }
    static constexpr bool isSync = false;

    WindowedPluginGeometryDidChange(const WebCore::IntRect& frameRect, const WebCore::IntRect& clipRect, uint64_t windowID)
        : m_arguments(frameRect, clipRect, windowID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(X11)
class WindowedPluginVisibilityDidChange {
public:
    using Arguments = std::tuple<bool, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_WindowedPluginVisibilityDidChange; }
    static constexpr bool isSync = false;

    WindowedPluginVisibilityDidChange(bool isVisible, uint64_t windowID)
        : m_arguments(isVisible, windowID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DidCreatePlugin {
public:
    using Arguments = std::tuple<bool, uint32_t>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_DidCreatePlugin; }
    static constexpr bool isSync = true;

    using DelayedReply = DidCreatePluginDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    DidCreatePlugin(bool wantsWheelEvents, uint32_t remoteLayerClientID)
        : m_arguments(wantsWheelEvents, remoteLayerClientID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFailToCreatePlugin {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_DidFailToCreatePlugin; }
    static constexpr bool isSync = true;

    using DelayedReply = DidFailToCreatePluginDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetPluginIsPlayingAudio {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::PluginProxy_SetPluginIsPlayingAudio; }
    static constexpr bool isSync = false;

    explicit SetPluginIsPlayingAudio(bool pluginIsPlayingAudio)
        : m_arguments(pluginIsPlayingAudio)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

} // namespace PluginProxy
} // namespace Messages

#endif // ENABLE(NETSCAPE_PLUGIN_API)
